From 9449f727348558f9f37f43d6d15cdced66db8450 Mon Sep 17 00:00:00 2001
From: Pierre d'Herbemont <pdherbemont@videolan.org>
Date: Fri, 16 May 2008 14:42:39 +0200
Subject: [PATCH] objects: Protect from FIND_PARENT directly in the FIND_PARENT cases.

This allows a correct vlc_object_detach(p_child) behaviour. Because vlc_object_detach() won't run if p_parent is set to NULL.
---
 src/misc/objects.c |   58 +++++++++++++++++++++++++++++++++++++++++++++------
 1 files changed, 51 insertions(+), 7 deletions(-)

diff --git a/src/misc/objects.c b/src/misc/objects.c
index 4c6547d..4bb546e 100644
--- a/src/misc/objects.c
+++ b/src/misc/objects.c
@@ -833,6 +833,37 @@ void __vlc_object_yield( vlc_object_t *p_this )
     vlc_spin_unlock( &internals->ref_spin );
 }
 
+/**
+ * Function used in the special FIND_PARENT case
+ * Do not use elsewhere
+ */
+static bool vlc_object_yield_if_not_destroyed( vlc_object_t *p_this )
+{
+    vlc_object_internals_t *internals = vlc_internals( p_this );
+
+    vlc_spin_lock( &internals->ref_spin );
+    /* Avoid obvious freed object uses */
+    assert( internals->i_refcount >= 0 );
+
+    /* Object is being destroyed don't do anything
+     * Caution, that object could be freed.
+     * Note, we use that fonction in a precise case
+     * where we are sure that the pointer is actually
+     * not freed.
+     */
+    if( internals->i_refcount == 0 )
+    {
+        vlc_spin_unlock( &internals->ref_spin );
+        return false;
+    }
+
+    /* Increment the counter */
+    internals->i_refcount++;
+    vlc_spin_unlock( &internals->ref_spin );
+
+    return true;
+}
+
 /*****************************************************************************
  * decrement an object refcount
  * And destroy the object if its refcount reach zero.
@@ -879,9 +910,8 @@ void __vlc_object_release( vlc_object_t *p_this )
         /* Detach from parent to protect against FIND_CHILDREN */
         if (p_this->p_parent)
             vlc_object_detach_unlocked (p_this);
-        /* Detach from children to protect against FIND_PARENT */
-        for (int i = 0; i < p_this->i_children; i++)
-            p_this->pp_children[i]->p_parent = NULL;
+        /* We will protect ourselves directly in FIND_PARENT
+         * for FIND_PARENT */
     }
 
     vlc_mutex_unlock( &structure_lock );
@@ -1271,9 +1301,16 @@ static vlc_object_t * FindObject( vlc_object_t *p_this, int i_type, int i_mode )
         p_tmp = p_this->p_parent;
         if( p_tmp )
         {
-            if( p_tmp->i_object_type == i_type )
+            if( p_tmp->i_object_type == i_type
+                /* Special case. If the parent object
+                 * is being destroyed (i_refcount == 0)
+                 * don't return it. Note, p_tmp can't
+                 * be invalid memory, as we assert
+                 * that each child is detached from
+                 * its parent before parent vlc_object_t
+                 * structure is freed */
+                && vlc_object_yield_if_not_destroyed( p_tmp ) )
             {
-                vlc_object_yield( p_tmp );
                 return p_tmp;
             }
             else
@@ -1325,9 +1362,16 @@ static vlc_object_t * FindObjectName( vlc_object_t *p_this,
         if( p_tmp )
         {
             if( p_tmp->psz_object_name
-                && !strcmp( p_tmp->psz_object_name, psz_name ) )
+                && !strcmp( p_tmp->psz_object_name, psz_name )
+                /* Special case. If the parent object
+                 * is being destroyed (i_refcount == 0)
+                 * don't return it. Note, p_tmp can't
+                 * be invalid memory, as we assert
+                 * that each child is detached from
+                 * its parent before parent vlc_object_t
+                 * structure is freed */
+                && vlc_object_yield_if_not_destroyed( p_tmp ) )
             {
-                vlc_object_yield( p_tmp );
                 return p_tmp;
             }
             else
-- 
1.5.4.4.481.g5075

